function [stimmat, stimparams] = getGratingFlashStimulus(stimdesc, spX, spY, bwflag)
% getGratingFlashStimulus - Generates a matrix of grating stimuli and corresponding parameters
%
% Syntax:  [stimmat, stimparams] = getGratingFlashStimulus(stimdesc, spX, spY, bwflag)
%
% Inputs:
%    stimdesc - struct containing information about the widths, phases, and orientations of the gratings
%    spX - the values of spatial x coordinates for which the grating is calculated
%    spY - the values of spatial y coordinates for which the grating is calculated
%    bwflag - marks whether the grating is square instead of sinusoidal 
%
% Outputs:
%    stimmat - [numel(spY) x numel(spX) x Nstimuli] matrix of grating stimuli, where the first two dimensions match the size of the spatial grid, and the third dimension corresponds to the number of different gratings generated.
%    stimparams - [Nstimuli x 3] matrix containing the parameters used to generate the gratings, including spatial frequency, orientation, and phase.

Nwidths   = numel(stimdesc.stripewidths);
Nstimuli  = sum(stimdesc.Nphases .* stimdesc.Norientations);

stimparams = zeros(Nstimuli, 3);
stimmat    = zeros(numel(spY) * numel(spX), Nstimuli, 'uint8');

[XX, YY] = meshgrid(spX, spY);

istim = 1;

for iw = 1:Nwidths
    
    spf   = 0.5/stimdesc.stripewidths(iw);
    nph   = stimdesc.Nphases(iw);
    nori  = stimdesc.Norientations(iw);
    
    for iori = 1:nori
        
        ortheta = pi*(iori-1)/nori;
        
        
        for iphase = 1:nph
            
            phangle = 2*pi*(iphase-1)/nph;
            vals = sin(2*pi*spf*(XX(:)*cos(ortheta) + YY(:)*sin(ortheta)) + phangle);
            if bwflag; vals = sign(vals); end
            stimmat(:, istim) = uint8(255 * (vals + 1)/2);
            stimparams(istim, :) = [spf ortheta phangle];
            istim = istim + 1;
            
        end 
    end
end

stimmat  = reshape(stimmat, numel(spY), numel(spX), Nstimuli);

end
